@@ -14,10 +14,10 @@
 #include <fit_em.h>
 #include <tree_model.h>
 #include <stringsplus.h>
-#include <string.h>
 #include <ctype.h>
 #include <numerical_opt.h>
 #include <tree_likelihoods.h>
+#include <subst_mods.h>
 #include <time.h>
 #include <sys/time.h>
 #include <sufficient_stats.h>
@@ -64,7 +64,8 @@ int tm_fit_em(TreeModel *mod, MSA *msa, Vector *params, int cat,
   if (msa->ss == NULL) {
     if (msa->seqs == NULL)
       die("ERROR tm_fit_em: msa->seqs == NULL\n");
-    ss_from_msas(msa, mod->order+1, 0, NULL, NULL, NULL, -1);
+    ss_from_msas(msa, mod->order+1, 0, NULL, NULL, NULL, -1, 
+		 subst_mod_is_codon_model(mod->subst_mod));
   }
 
   if (mod->backgd_freqs == NULL) { 
@@ -108,7 +109,7 @@ int tm_fit_em(TreeModel *mod, MSA *msa, Vector *params, int cat,
      when close to convergence */
   nratecats = mod->nratecats;
   if (nratecats > 1) {
-    old_param_map = malloc(params->size*sizeof(int));
+    old_param_map = smalloc(params->size*sizeof(int));
     for (i=0; i<params->size; i++)
       old_param_map[i] = mod->param_map[i];
     nratevarparams = tm_get_nratevarparams(mod);
@@ -128,7 +129,9 @@ int tm_fit_em(TreeModel *mod, MSA *msa, Vector *params, int cat,
   grad_func = (proj_mod == NULL && 
                mod->estimate_branchlens == TM_BRANCHLENS_ALL && 
                mod->subst_mod != JC69 && mod->subst_mod != F81 &&
-               !mod->estimate_backgd ? compute_grad_em_approx : NULL);
+               !mod->estimate_backgd && mod->alt_subst_mods==NULL &&
+	       mod->selection_idx < 0 ? 
+	       compute_grad_em_approx : NULL);
                                 /* functions for analytical gradients
                                    don't yet know about estimating
                                    scale or backgd freqs, also require
@@ -147,16 +150,17 @@ int tm_fit_em(TreeModel *mod, MSA *msa, Vector *params, int cat,
   }
 
   /* most params have lower bound of zero and no upper bound */
-  lower_bounds = vec_new(npar);
+  /*  lower_bounds = vec_new(npar);
   vec_zero(lower_bounds);
-  upper_bounds = NULL;
+  upper_bounds = NULL;*/
 
   /* however, in this case we don't want the eq freqs to go to zero */
-  if (mod->estimate_backgd) {
+  /*  if (mod->estimate_backgd) {
     for (i = 0; i < mod->backgd_freqs->size; i++)
       if (mod->param_map[mod->backgd_idx+i] >= 0)  //this should be true since mod->estimate_backgd
 	vec_set(lower_bounds, mod->param_map[mod->backgd_idx + i], 0.001);
-  }
+	}*/
+  tm_set_boundaries(&lower_bounds, &upper_bounds, npar, mod);
 
 
   H = mat_new(npar, npar);
@@ -309,7 +313,7 @@ int tm_fit_em(TreeModel *mod, MSA *msa, Vector *params, int cat,
   vec_free(lower_bounds);
   tl_free_tree_posteriors(mod, msa, mod->tree_posteriors);
   mod->tree_posteriors = NULL;
-  if (old_param_map != NULL) free(old_param_map);
+  if (old_param_map != NULL) sfree(old_param_map);
 
   vec_free(opt_params);
   return retval;
@@ -401,12 +405,16 @@ void compute_grad_em_approx(Vector *grad, Vector *params, void *data,
     **q3dq = NULL;
   static Complex *diag = NULL;
 
+  if  (Q->evals_z == NULL || Q->evec_matrix_z == NULL || Q->evec_matrix_inv_z == NULL)
+    die("ERRROR: compute_grad_em_approx got NULL value in eigensystem; error diagonalizing matrix.");
+
   if (diag == NULL) 
     diag = (Complex*)smalloc(nstates * sizeof(Complex));
 
   /* init memory (first time only) */
   if (q == NULL) {
     q = (double**)smalloc(nstates * sizeof(double*));
+    set_static_var((void**)&q);
     q2 = (double**)smalloc(nstates * sizeof(double*));
     q3 = (double**)smalloc(nstates * sizeof(double*));
     dq = (double**)smalloc(nstates * sizeof(double*));
@@ -497,7 +505,7 @@ void compute_grad_em_approx(Vector *grad, Vector *params, void *data,
       /* save time by only using complex numbers in the inner loop if
          necessary (each complex mult equivalent to four real mults and
          two real adds) */
-      if (tm_is_reversible(mod->subst_mod)) {
+      if (tm_node_is_reversible(mod, n)) {
         for (k = 0; k < nstates; k++) {
           for (l = 0; l < nstates; l++) {
             double p = mm_get(P, k, l);
@@ -586,7 +594,7 @@ void compute_grad_em_approx(Vector *grad, Vector *params, void *data,
           diag[i] = z_mul_real(z_mul(z_exp(z_mul_real(zvec_get(Q->evals_z, i), t)), zvec_get(Q->evals_z, i)), n->dparent * dr_da);
 
         /* only use complex numbers if necessary (as above) */
-        if (tm_is_reversible(mod->subst_mod)) {
+        if (tm_node_is_reversible(mod, n)) {
           for (k = 0; k < nstates; k++) {
             for (l = 0; l < nstates; l++) {
               double p = mm_get(P, k, l);
@@ -696,7 +704,7 @@ void compute_grad_em_approx(Vector *grad, Vector *params, void *data,
       if (dq[l][m] != 0)    /* row/col pairs should be unique */
 	die("ERROR compute_grad_em_approx: dq[%i][%i] should be zero but is %e\n", l, m, dq[l][m]);
 
-      dq[l][m] = tm_is_reversible(mod->subst_mod) ? 
+      dq[l][m] = subst_mod_is_reversible(mod->subst_mod) ? 
         vec_get(mod->backgd_freqs, m) : 1;
                                 /* FIXME: may need to generalize */
       
@@ -876,14 +884,20 @@ void compute_grad_em_exact(Vector *grad, Vector *params, void *data,
   static Complex **f = NULL, **tmpmat = NULL, **sinv_dq_s = NULL;
   static Complex *diag = NULL;
 
-  if (diag == NULL) 
+  if (diag == NULL) {
     diag = (Complex*)smalloc(nstates * sizeof(Complex));
+    set_static_var((void**)&diag);
+  }
 
   Q = mod->rate_matrix;
+  if (Q->evals_z == NULL || Q->evec_matrix_z == NULL ||
+      Q->evec_matrix_inv_z == NULL)
+    die("ERROR compute_grade_em_exact got NULL value in eigensystem; error diagonalizing rate matrix\n");
 
   /* init memory (first time only) */
   if (dq == NULL) {
     dq = (double**)smalloc(nstates * sizeof(double*));
+    set_static_var((void**)&dq);
     f = (Complex**)smalloc(nstates * sizeof(Complex*));
     tmpmat = (Complex**)smalloc(nstates * sizeof(Complex*));
     sinv_dq_s = (Complex**)smalloc(nstates * sizeof(Complex*));
@@ -928,7 +942,7 @@ void compute_grad_em_exact(Vector *grad, Vector *params, void *data,
       /* save time by only using complex numbers in the inner loop if
          necessary (each complex mult equivalent to four real mults and
          two real adds) */
-      if (tm_is_reversible(mod->subst_mod)) {
+      if (tm_node_is_reversible(mod, n)) {
         for (k = 0; k < nstates; k++) {
           for (l = 0; l < nstates; l++) {
             double p = mm_get(P, k, l);
@@ -1013,7 +1027,7 @@ void compute_grad_em_exact(Vector *grad, Vector *params, void *data,
 	    diag[i] = z_mul_real(z_mul(z_exp(z_mul_real(zvec_get(Q->evals_z, i), t)), zvec_get(Q->evals_z, i)), n->dparent * dr_da);
 	  
 	  /* only use complex numbers if necessary (as above) */
-	  if (tm_is_reversible(mod->subst_mod)) {
+	  if (tm_node_is_reversible(mod, n)) {
 	    for (k = 0; k < nstates; k++) {
 	      for (l = 0; l < nstates; l++) {
 		double p = mm_get(P, k, l);
@@ -1126,7 +1140,7 @@ void compute_grad_em_exact(Vector *grad, Vector *params, void *data,
 	die("ERROR compute_grad_exact dq[%i][%i] should be zero but is %e\n",
 	    l, m, dq[l][m]);
 
-      dq[l][m] = tm_is_reversible(mod->subst_mod) ? 
+      dq[l][m] = subst_mod_is_reversible(mod->subst_mod) ? 
         vec_get(mod->backgd_freqs, m) : 1;
                                 /* FIXME: may need to generalize */
       
@@ -1182,7 +1196,7 @@ void compute_grad_em_exact(Vector *grad, Vector *params, void *data,
         /* as above, it's worth it to have separate versions of the
            computations below for the real and complex cases */
 
-        if (tm_is_reversible(mod->subst_mod)) { /* real case */
+        if (tm_node_is_reversible(mod, n)) { /* real case */
           /* build the matrix F */
           for (i = 0; i < nstates; i++) {
             for (j = 0; j < nstates; j++) {
