@@ -28,7 +28,6 @@
 #include <tree_model.h>
 #include <fit_em.h>
 #include <subst_mods.h>
-#include <string.h>
 #include <local_alignment.h>
 #include <ctype.h>
 #include <tree_likelihoods.h>
@@ -47,7 +46,7 @@ struct phyloFit_struct* phyloFit_struct_new(int rphast) {
   struct phyloFit_struct *pf = smalloc(sizeof(struct phyloFit_struct));
   pf->msa = NULL;
   pf->output_fname_root = rphast ? NULL : "phyloFit";
-  pf->log_fname = NULL;
+  pf->logf = NULL;
   pf->reverse_group_tag = NULL;
   pf->root_seqname = NULL;
   pf->subtree_name = NULL;
@@ -68,12 +67,13 @@ struct phyloFit_struct* phyloFit_struct_new(int rphast) {
   pf->do_expected_nsubst = FALSE;
   pf->do_expected_nsubst_tot = FALSE;
   pf->random_init = FALSE;
-  pf->estimate_backgd = FALSE;
+  pf->estimate_backgd = rphast ? TRUE : FALSE;  //in rphast mode we use no.opt to specify no estimate backgd
   pf->estimate_scale_only = FALSE;
   pf->do_column_probs = FALSE;
   pf->nonoverlapping = FALSE;
   pf->gaps_as_bases = FALSE;
   pf->no_freqs = FALSE;
+  pf->init_backgd_from_data = TRUE;
   pf->no_rates = FALSE;
   pf->assume_clock = FALSE;
   pf->init_parsimony = FALSE;
@@ -90,83 +90,20 @@ struct phyloFit_struct* phyloFit_struct_new(int rphast) {
   pf->window_coords=NULL;
   pf->ignore_branches = NULL;
   pf->alt_mod_str = NULL;
+  pf->label_str = NULL;
+  pf->label_type = NULL;
   pf->bound_arg = NULL;
   pf->rate_consts = NULL;
   pf->alpha = DEFAULT_ALPHA;
   pf->gff = NULL;
   pf->input_mod = NULL;
+  pf->use_selection = 0;
+  pf->selection = 0.0;
   
-  pf->estimated_models = NULL;
-  pf->model_labels = NULL;
+  pf->results = rphast ? lol_new(2) : NULL;
   return pf;
 }
 
-//print contents of phyloFit struct for testing
-void print_phyloFit_struct(struct phyloFit_struct *pf) {
-  printf("msa->length=%i\n", pf->msa->length);
-  printf("msa->nseqs=%i\n", pf->msa->nseqs);
-  printf("output_fname_root=%s\n", pf->output_fname_root==NULL ? "NULL" : pf->output_fname_root);
-  printf("log_fname = %s\n", pf->log_fname == NULL ? "NULL" : pf->log_fname);
-  printf("reverse_group_tag = %s\n", pf->reverse_group_tag == NULL ? "NULL" : pf->reverse_group_tag);
-  printf("root_seqname = %s\n", pf->root_seqname == NULL ? "NULL" : pf->root_seqname);
-  printf("subtree_name = %s\n", pf->subtree_name == NULL ? "NULL" : pf->subtree_name);
-  printf("error_fname = %s\n", pf->error_fname == NULL ? "NULL" : pf->error_fname);
-  printf("see_for_help = %s\n", pf->see_for_help == NULL ? "NULL" : pf->see_for_help);
-  printf("parsimony_cost_fname = %s\n", pf->parsimony_cost_fname == NULL ? "NULL" : pf->parsimony_cost_fname);
-  printf("msa_fname = %s\n", pf->msa_fname == NULL ? "NULL" : pf->msa_fname);
-  printf("subst_mod = %i\n", pf->subst_mod);
-  printf("quiet = %i\n", pf->quiet);
-  printf("nratecats = %i\n", pf->nratecats);
-  printf("use_em = %i\n", pf->use_em);
-  printf("window_size = %i\n", pf->window_size);
-  printf("window_shift = %i\n", pf->window_shift);
-  printf("use_Conditionals=%i\n", pf->use_conditionals);
-  printf("precision = %i\n", pf->precision);
-  printf("likelihood_only = %i\n", pf->likelihood_only);
-  printf("do_bases = %i\n", pf->do_bases);
-  printf("do_expected_nsubst = %i\n", pf->do_expected_nsubst);
-  printf("do_expected_nsubst_tot = %i\n", pf->do_expected_nsubst_tot);
-  printf("random_init = %i\n", pf->random_init);
-  printf("estimate_backgd = %i\n", pf->estimate_backgd);
-  printf("estimate_scale_only=%i\n", pf->estimate_scale_only);
-  printf("do_column_probs = %i\n", pf->do_column_probs);
-  printf("nonoverlapping = %i\n", pf->nonoverlapping);
-  printf("gaps_as_bases = %i\n", pf->gaps_as_bases);
-  printf("no_freqs = %i\n", pf->no_freqs);
-  printf("no_rates = %i\n", pf->no_rates);
-  printf("assume_clock=%i\n", pf->assume_clock);
-  printf("init_parsimony = %i\n", pf->init_parsimony);
-  printf("parimony_only = %i\n", pf->parsimony_only);
-  printf("no_branchlens = %i\n", pf->no_branchlens);
-  printf("label_categories = %i\n", pf->label_categories);
-  printf("nsites_threshold=%i\n", pf->nsites_threshold);
-  printf("symfreq = %i\n", pf->symfreq);
-  printf("tree: ");
-  if (pf->tree != NULL)
-    tr_print(stdout, pf->tree, 1);
-  else printf("NULL\n");
-  printf("cm: ");
-  if (pf->cm == NULL) printf("is null\n");
-  else cm_print(pf->cm, stdout);
-  printf("nooptstr = %s\n", pf->nooptstr==NULL ? "NULL" : pf->nooptstr->chars);
-  printf("cats_to_do==NULL=%i\n", pf->cats_to_do_str==NULL);
-  printf("window_coords==NULL=%i\n", pf->window_coords==NULL);
-  printf("ignore_branches==NULL=%i\n", pf->ignore_branches==NULL);
-  printf("alt_mod_str==NULL=%i\n", pf->alt_mod_str==NULL);
-  printf("bound_arg==NULL=%i\n", pf->bound_arg==NULL);
-  printf("rate_consts==NULL=%i\n", pf->rate_consts==NULL);
-  printf("alpha = %f\n", pf->alpha);
-  printf("gff is NULL=%i\n", pf->gff==NULL);
-  if (pf->input_mod != NULL)
-    tm_print(stdout, pf->input_mod);
-  else printf("NULL\n");
-
-  printf("estimated_models==NULL=%i\n", pf->estimated_models==NULL);
-  printf("model_labels == NULL=%i\n", pf->model_labels==NULL);
-}
-
-
-
 
 void set_output_fname(String *fname, char *root, int cat, char *suffix) {
   str_cpy_charstr(fname, root);
@@ -178,6 +115,85 @@ void set_output_fname(String *fname, char *root, int cat, char *suffix) {
 }
 
 
+
+int add_ratecat_names_to_lol(ListOfLists *lol, TreeModel *mod) {
+  List *l = lst_new_ptr(mod->nratecats);
+  int i;
+  char *tempstr1, tempstr2[100];
+  for (i=0; i < mod->nratecats; i++) {
+    sprintf(tempstr2, "rate.cat.%i", i);
+    tempstr1 = smalloc((strlen(tempstr2)+1)*sizeof(char));
+    strcpy(tempstr1, tempstr2);
+    lst_push_ptr(l, tempstr1);
+  }
+  lol_push_list(lol, l, NULL, CHAR_LIST);
+  return lst_size(l);
+}
+
+
+int add_node_names_to_lol(ListOfLists *lol, TreeModel *mod, 
+			  int include_leaf_branches, int include_root,
+			  int add_count_column) {
+  List *l = lst_new_ptr(mod->tree->nnodes + add_count_column);
+  int node;
+  TreeNode *n;
+  char *tempstr;
+  if (add_count_column) {
+    tempstr = smalloc(6*sizeof(char));
+    strcpy(tempstr, "nsite");
+    lst_push_ptr(l, tempstr);
+  }
+  tr_name_ancestors(mod->tree);
+  for (node = 0; node < mod->tree->nnodes; node++) {
+    n = lst_get_ptr(mod->tree->nodes, node);
+    if (n == mod->tree && !include_root) continue;
+    if ((n->lchild == NULL || n->rchild == NULL) && !include_leaf_branches)
+      continue;
+    tempstr = smalloc((strlen(n->name)+1)*sizeof(char));
+    strcpy(tempstr, n->name);
+    lst_push_ptr(l, tempstr);
+  }
+  lol_push_list(lol, l, NULL, CHAR_LIST);
+  return lst_size(l);
+}
+
+int add_state_names_to_lol(ListOfLists *lol, TreeModel *mod, 
+			   const char *prefix) {
+  char *tempstr;
+  List *l = lst_new_ptr(mod->rate_matrix->size);
+  int state, prefixlen;
+  if (prefix == NULL) prefixlen = 0;
+  else prefixlen = strlen(prefix);
+  for (state=0; state < mod->rate_matrix->size; state++) {
+    tempstr = smalloc((mod->order + 2 + prefixlen)*sizeof(char));
+    tempstr[mod->order + 1 + prefixlen] = '\0';
+    if (prefix != NULL) strcpy(tempstr, prefix);
+    get_tuple_str(tempstr+prefixlen, state, mod->order + 1, 
+		  mod->rate_matrix->states);
+    lst_push_ptr(l, tempstr);
+  }
+  lol_push_list(lol, l, NULL, CHAR_LIST);
+  return lst_size(l);
+}
+
+int add_tuple_names_to_lol(ListOfLists *lol, TreeModel *mod, 
+			   MSA *msa, int cat) {
+  List *l = lst_new_ptr(msa->ss->ntuples);
+  int tup;
+  char *tempstr;
+  for (tup = 0; tup < msa->ss->ntuples; tup++) {
+    checkInterruptN(tup, 1000);
+    if ((cat >= 0 && msa->ss->cat_counts[cat][tup] == 0) ||
+	msa->ss->counts[tup] == 0) continue;
+    tempstr = smalloc((msa->nseqs+1)*sizeof(char));
+    tuple_to_string_pretty(tempstr, msa, tup);
+    lst_push_ptr(l, tempstr);
+  }
+  lol_push_list(lol, l, NULL, CHAR_LIST);
+  return lst_size(l);
+}
+
+
 /* Compute and output statistics based on posterior probabilities,
    including (optionally) the post prob of each tuple of bases at each
    ancestral node at each site (do_bases), the expected total number
@@ -188,19 +204,22 @@ void set_output_fname(String *fname, char *root, int cat, char *suffix) {
    ".expsub", and ".exptotsub", respectively).    */
 void print_post_prob_stats(TreeModel *mod, MSA *msa, char *output_fname_root, 
                            int do_bases, int do_expected_nsubst, 
-                           int do_expected_nsubst_tot, int cat, int quiet) {
+                           int do_expected_nsubst_tot, int cat, int quiet,
+			   ListOfLists *results) {
   String *fname = str_new(STR_MED_LEN);
   FILE *POSTPROBF, *EXPSUBF, *EXPTOTSUBF;
   int i, tup, node, state, state2;
   TreeNode *n;
   char tuplestr[mod->order+2];
   char coltupstr[msa->nseqs+1];
+  int ratecat;
+  ListOfLists *dobase_lol = NULL, *exp_nsub_lol=NULL, *exp_nsub_tot_lol=NULL;
   tuplestr[mod->order+1] = '\0';
   coltupstr[msa->nseqs] = '\0';
 
   /* FIXME: rate variation!     need rate post probs! */
   if (mod->nratecats != 1)
-    die("ERROR print_post_prob_states nratecats should be 1 but is %i\n",
+    die("ERROR print_post_prob_stats nratecats should be 1 but is %i\n",
 	mod->nratecats);
 
   /* compute desired stats */
@@ -212,172 +231,322 @@ void print_post_prob_stats(TreeModel *mod, MSA *msa, char *output_fname_root,
                                                 do_expected_nsubst, 
                                                 do_expected_nsubst_tot, 0, 0);
   tl_compute_log_likelihood(mod, msa, NULL, cat, mod->tree_posteriors);
+  tr_name_ancestors(mod->tree);
 
   if (do_bases) {
-    set_output_fname(fname, output_fname_root, cat, ".postprob");
-    if (!quiet) 
-      fprintf(stderr, "Writing posterior probabilities to %s ...\n", 
-              fname->chars);
-    POSTPROBF = fopen_fname(fname->chars, "w+");
+    if (results != NULL) {
+      ListOfLists *dimnames_lol = lol_new(4);
+      int *dim = smalloc(4*sizeof(int));
+      double *temparr;
+      
+      dim[0] = add_ratecat_names_to_lol(dimnames_lol, mod);
+      dim[1] = add_node_names_to_lol(dimnames_lol, mod, 0, 1, 0);
+      dim[2] = add_state_names_to_lol(dimnames_lol, mod, NULL);
+      dim[3] = add_tuple_names_to_lol(dimnames_lol, mod, msa, cat);
 
-    /* print header */
-    fprintf(POSTPROBF, "%-6s ", "#");
-    for (i = 0; i < msa->nseqs; i++) fprintf(POSTPROBF, " ");
-    fprintf(POSTPROBF, "    ");
-    for (node = 0; node < mod->tree->nnodes; node++) {
-      n = lst_get_ptr(mod->tree->nodes, node);
-      if (n->lchild == NULL || n->rchild == NULL) continue;
-      for (state = 0; state < mod->rate_matrix->size; state++) {
-        if (state == mod->rate_matrix->size/2)
-          fprintf(POSTPROBF, "node %-2d", n->id);
-        else
-          fprintf(POSTPROBF, "%6s ", "");
+
+      dobase_lol = lol_new(2 + dim[0]);
+      lol_set_class(dobase_lol, "array");
+      lol_push_lol(dobase_lol, dimnames_lol, "dimnames");
+      lol_push_int(dobase_lol, dim, 4, "dim");
+
+      temparr = smalloc(dim[3]*sizeof(double));
+      for (ratecat=0; ratecat < mod->nratecats; ratecat++) {
+	ListOfLists *ratecat_lol = lol_new(dim[1]);
+	for (node = 0; node < mod->tree->nnodes; node++) {
+	  ListOfLists *node_lol;
+	  n = (TreeNode*)lst_get_ptr(mod->tree->nodes, node);
+	  if (n->lchild == NULL || n->rchild == NULL) continue;
+	  node_lol = lol_new(dim[2]);
+	  for (state = 0 ; state < mod->rate_matrix->size; state++) {
+	    int idx=0;
+	    for (tup=0; tup < msa->ss->ntuples; tup++) {
+	      if ((cat >=0 && msa->ss->cat_counts[cat][tup] == 0) ||
+		  msa->ss->counts[tup] == 0) continue;
+	      temparr[idx++] = mod->tree_posteriors->base_probs[ratecat][state][node][tup];
+	    }
+	    lol_push_dbl(node_lol, temparr, dim[3], NULL);
+	  }
+	  lol_push_lol(ratecat_lol, node_lol, NULL);
+	}
+	lol_push_lol(dobase_lol, ratecat_lol, NULL);
       }
+      sfree(temparr);
+      sfree(dim);
     }
-    fprintf(POSTPROBF, "\n%-6s ", "#");
-    for (state = 0; state < msa->nseqs-5; state++) fprintf(POSTPROBF, " ");
-    fprintf(POSTPROBF, "tuple     ");
-    for (node = 0; node < mod->tree->nnodes; node++) {
-      n = lst_get_ptr(mod->tree->nodes, node);
-      if (n->lchild == NULL || n->rchild == NULL) continue;
-      for (state = 0; state < mod->rate_matrix->size; state++) {
-        get_tuple_str(tuplestr, state, mod->order + 1, 
-                      mod->rate_matrix->states);
-        fprintf(POSTPROBF, "%6s ", tuplestr);
-      }
-    }
-    fprintf(POSTPROBF, "\n");
-
-    /* print post probs */
-    for (tup = 0; tup < msa->ss->ntuples; tup++) {
-      checkInterruptN(tup, 1000);
-
-      if ((cat >= 0 && msa->ss->cat_counts[cat][tup] == 0) ||
-          msa->ss->counts[tup] == 0) continue;
-
-      tuple_to_string_pretty(coltupstr, msa, tup);
-      fprintf(POSTPROBF, "%-6d %5s      ", tup, coltupstr);
+    if (output_fname_root != NULL) {
+      set_output_fname(fname, output_fname_root, cat, ".postprob");
+      if (!quiet) 
+	fprintf(stderr, "Writing posterior probabilities to %s ...\n", 
+		fname->chars);
+      POSTPROBF = fopen_fname(fname->chars, "w+");
+      
+      /* print header */
+      fprintf(POSTPROBF, "%-6s ", "#");
+      for (i = 0; i < msa->nseqs; i++) fprintf(POSTPROBF, " ");
+      fprintf(POSTPROBF, "    ");
       for (node = 0; node < mod->tree->nnodes; node++) {
-        n = lst_get_ptr(mod->tree->nodes, node);
-        if (n->lchild == NULL || n->rchild == NULL) continue;
-        for (state = 0; state < mod->rate_matrix->size; state++) 
-          fprintf(POSTPROBF, "%6.4f ", 
-                  mod->tree_posteriors->base_probs[0][state][node][tup]);
-      }                 
+	n = lst_get_ptr(mod->tree->nodes, node);
+	if (n->lchild == NULL || n->rchild == NULL) continue;
+	for (state = 0; state < mod->rate_matrix->size; state++) {
+	  if (state == mod->rate_matrix->size/2)
+	    fprintf(POSTPROBF, "node %-2d", n->id);
+	  else
+	    fprintf(POSTPROBF, "%6s ", "");
+	}
+      }
+      fprintf(POSTPROBF, "\n%-6s ", "#");
+      for (state = 0; state < msa->nseqs-5; state++) fprintf(POSTPROBF, " ");
+      fprintf(POSTPROBF, "tuple     ");
+      for (node = 0; node < mod->tree->nnodes; node++) {
+	n = lst_get_ptr(mod->tree->nodes, node);
+	if (n->lchild == NULL || n->rchild == NULL) continue;
+	for (state = 0; state < mod->rate_matrix->size; state++) {
+	  get_tuple_str(tuplestr, state, mod->order + 1, 
+			mod->rate_matrix->states);
+	  fprintf(POSTPROBF, "%6s ", tuplestr);
+	}
+      }
       fprintf(POSTPROBF, "\n");
+      
+      /* print post probs */
+      for (tup = 0; tup < msa->ss->ntuples; tup++) {
+	checkInterruptN(tup, 1000);
+	
+	if ((cat >= 0 && msa->ss->cat_counts[cat][tup] == 0) ||
+	    msa->ss->counts[tup] == 0) continue;
+	
+	tuple_to_string_pretty(coltupstr, msa, tup);
+	fprintf(POSTPROBF, "%-6d %5s      ", tup, coltupstr);
+	for (node = 0; node < mod->tree->nnodes; node++) {
+	  n = lst_get_ptr(mod->tree->nodes, node);
+	  if (n->lchild == NULL || n->rchild == NULL) continue;
+	  for (state = 0; state < mod->rate_matrix->size; state++) 
+	    fprintf(POSTPROBF, "%6.4f ", 
+		    mod->tree_posteriors->base_probs[0][state][node][tup]);
+	}                 
+	fprintf(POSTPROBF, "\n");
+      }
+      fclose(POSTPROBF);
     }
-    fclose(POSTPROBF);
   }
 
   if (do_expected_nsubst) {
-    set_output_fname(fname, output_fname_root, cat, ".expsub");
-    if (!quiet) 
-      fprintf(stderr, "Writing expected numbers of substitutions to %s ...\n", 
-              fname->chars);
-    EXPSUBF = fopen_fname(fname->chars, "w+");
+    if (results != NULL) {
+      ListOfLists *dimnames_lol = lol_new(3);
+      int *dim = smalloc(3*sizeof(int));
+      double *temparr, *countarr;
+      int *icountarr;
 
-    fprintf(EXPSUBF, "%-3s %10s %7s ", "#", "tuple", "count");
-    for (node = 0; node < mod->tree->nnodes; node++) {
-      n = lst_get_ptr(tr_postorder(mod->tree), node);
-      if (n == mod->tree) continue;
-      fprintf(EXPSUBF, " node_%-2d", n->id);
+      dim[0] = add_ratecat_names_to_lol(dimnames_lol, mod);
+      dim[1] = add_node_names_to_lol(dimnames_lol, mod, 1, 0, 1);
+      dim[2] = add_tuple_names_to_lol(dimnames_lol, mod, msa, cat);
+
+      exp_nsub_lol = lol_new(2 + dim[0]);
+      lol_set_class(exp_nsub_lol, "array");
+      lol_push_lol(exp_nsub_lol, dimnames_lol, "dimnames");
+      lol_push_int(exp_nsub_lol, dim, 3, "dim");
+
+      temparr = smalloc(dim[2]*sizeof(double));
+      countarr = smalloc(dim[2]*sizeof(double));
+      icountarr = smalloc(dim[2]*sizeof(int));
+      for (ratecat=0; ratecat < mod->nratecats; ratecat++) {
+	ListOfLists *ratecat_lol = lol_new(dim[1]);
+	int node_idx=0;
+	for (node=0; node < mod->tree->nnodes; node++) {
+	  int idx=0;
+	  n = lst_get_ptr(mod->tree->nodes, node);
+	  if (n == mod->tree) continue;
+	  for (tup=0; tup < msa->ss->ntuples; tup++) {
+	    checkInterruptN(tup, 1000);
+	    if ((cat >= 0 && msa->ss->cat_counts[cat][tup] == 0) ||
+		msa->ss->counts[tup] == 0) continue;
+	    if (cat >=0) countarr[idx] = (int)msa->ss->cat_counts[cat][tup];
+	    else countarr[idx] = (int)msa->ss->counts[tup];
+	    temparr[idx++] = mod->tree_posteriors->expected_nsubst[ratecat][n->id][tup];
+	  }
+	  if (node_idx==0) {
+	    for (i=0; i < dim[2]; i++) {
+	      icountarr[i] = (int)countarr[i];
+	      if (fabs(countarr[i] - (double)icountarr[i]) > 1.0e-6) break;
+	    }
+	    if (i == dim[2]) 
+	      lol_push_int(ratecat_lol, icountarr, dim[2], NULL);
+	    else lol_push_dbl(ratecat_lol, countarr, dim[2], NULL);
+	  }
+	  lol_push_dbl(ratecat_lol, temparr, dim[2], NULL);
+	  node_idx++;
+	}
+	lol_push_lol(exp_nsub_lol, ratecat_lol, NULL);
+      }
+      sfree(countarr);
+      sfree(temparr);
+      sfree(dim);
     }
-    fprintf(EXPSUBF, "    total\n");
-    for (tup = 0; tup < msa->ss->ntuples; tup++) {
-      double total = 0;
-      checkInterruptN(tup, 1000);
-
-      if ((cat >= 0 && msa->ss->cat_counts[cat][tup] == 0) ||
-          msa->ss->counts[tup] == 0) continue;
-
-      tuple_to_string_pretty(coltupstr, msa, tup);
-      fprintf(EXPSUBF, "%-3d %10s %.0f ", tup, coltupstr, msa->ss->counts[tup]);
+    if (output_fname_root != NULL) {
+      set_output_fname(fname, output_fname_root, cat, ".expsub");
+      if (!quiet) 
+	fprintf(stderr, "Writing expected numbers of substitutions to %s ...\n", 
+		fname->chars);
+      EXPSUBF = fopen_fname(fname->chars, "w+");
+      
+      fprintf(EXPSUBF, "%-3s %10s %7s ", "#", "tuple", "count");
       for (node = 0; node < mod->tree->nnodes; node++) {
-        n = lst_get_ptr(tr_postorder(mod->tree), node);
-        if (n == mod->tree) continue;
-        fprintf(EXPSUBF, "%7.4f ", 
-                mod->tree_posteriors->expected_nsubst[0][n->id][tup]);
-        total += mod->tree_posteriors->expected_nsubst[0][n->id][tup];
-      }                 
-      fprintf(EXPSUBF, "%7.4f\n", total);
+	n = lst_get_ptr(tr_postorder(mod->tree), node);
+	if (n == mod->tree) continue;
+	fprintf(EXPSUBF, " node_%-2d", n->id);
+      }
+      fprintf(EXPSUBF, "    total\n");
+      for (tup = 0; tup < msa->ss->ntuples; tup++) {
+	double total = 0;
+	checkInterruptN(tup, 1000);
+	
+	if ((cat >= 0 && msa->ss->cat_counts[cat][tup] == 0) ||
+	    msa->ss->counts[tup] == 0) continue;
+	
+	tuple_to_string_pretty(coltupstr, msa, tup);
+	fprintf(EXPSUBF, "%-3d %10s %.0f ", tup, coltupstr, msa->ss->counts[tup]);
+	for (node = 0; node < mod->tree->nnodes; node++) {
+	  n = lst_get_ptr(tr_postorder(mod->tree), node);
+	  if (n == mod->tree) continue;
+	  fprintf(EXPSUBF, "%7.4f ", 
+		  mod->tree_posteriors->expected_nsubst[0][n->id][tup]);
+	  total += mod->tree_posteriors->expected_nsubst[0][n->id][tup];
+	}                 
+	fprintf(EXPSUBF, "%7.4f\n", total);
+      }
+      fclose(EXPSUBF);
     }
-    fclose(EXPSUBF);
   }
 
   if (do_expected_nsubst_tot) {
-    set_output_fname(fname, output_fname_root, cat, ".exptotsub");
-    if (!quiet) 
-      fprintf(stderr, "Writing total expected numbers of substitutions to %s ...\n", 
-              fname->chars);
-    EXPTOTSUBF = fopen_fname(fname->chars, "w+");
+    if (results != NULL) {
+      ListOfLists *dimnames_lol = lol_new(4);
+      int *dim = smalloc(4*sizeof(int));
+      double *temparr;
 
-    fprintf(EXPTOTSUBF, "\n\
+      dim[0] = add_ratecat_names_to_lol(dimnames_lol, mod);
+      dim[1] = add_node_names_to_lol(dimnames_lol, mod, 1, 0, 0);
+      dim[2] = add_state_names_to_lol(dimnames_lol, mod, "from.");
+      dim[3] = add_state_names_to_lol(dimnames_lol, mod, "to.");
+
+      exp_nsub_tot_lol = lol_new(2 + dim[0]);
+      lol_set_class(exp_nsub_tot_lol, "array");
+      lol_push_lol(exp_nsub_tot_lol, dimnames_lol, "dimnames");
+      lol_push_int(exp_nsub_tot_lol, dim, 4, "dim");
+
+      temparr = smalloc(dim[3]*sizeof(double));
+      for (ratecat=0; ratecat < mod->nratecats; ratecat++) {
+	ListOfLists *ratecat_lol = lol_new(dim[1]);
+	for (node = 0 ; node < mod->tree->nnodes; node++) {
+	  ListOfLists *node_lol = lol_new(dim[2]);
+	  n = lst_get_ptr(mod->tree->nodes, node);
+	  if (n == mod->tree) continue;
+	  for (state=0; state < mod->rate_matrix->size; state++) {
+	    for (state2=0; state2 < mod->rate_matrix->size; state2++) 
+	      temparr[state2] = mod->tree_posteriors->expected_nsubst_tot[0][state][state2][n->id];
+	    lol_push_dbl(node_lol, temparr, mod->rate_matrix->size, NULL);
+	  }
+	  lol_push_lol(ratecat_lol, node_lol, NULL);
+	}
+	lol_push_lol(exp_nsub_tot_lol, ratecat_lol, NULL);
+      }
+    }
+    if (output_fname_root != NULL) {
+      set_output_fname(fname, output_fname_root, cat, ".exptotsub");
+      if (!quiet) 
+	fprintf(stderr, "Writing total expected numbers of substitutions to %s ...\n", 
+		fname->chars);
+      EXPTOTSUBF = fopen_fname(fname->chars, "w+");
+      
+      fprintf(EXPTOTSUBF, "\n\
 A separate matrix of expected numbers of substitutions is shown for each\n\
 branch of the tree.     Nodes of the tree are visited in a postorder traversal,\n\
 and each node is taken to be representative of the branch between itself and\n\
 its parent.     Starting bases or tuples of bases appear on the vertical axis\n\
 of each matrix, and destination bases or tuples of bases appear on the\n\
 horizontal axis.\n\n");
-
-    for (node = 0; node < mod->tree->nnodes; node++) {
-      n = lst_get_ptr(tr_postorder(mod->tree), node);
-      if (n == mod->tree) continue;
-
-      fprintf(EXPTOTSUBF, "Branch above node %d", n->id);
-      if (n->name != NULL && strlen(n->name) > 0) 
-        fprintf(EXPTOTSUBF, " (leaf labeled '%s')", n->name);
-      fprintf(EXPTOTSUBF, ":\n\n");
-
-      /* print header */
-      fprintf(EXPTOTSUBF, "%-4s ", "");
-      for (state2 = 0; state2 < mod->rate_matrix->size; state2++) {
-        get_tuple_str(tuplestr, state2, mod->order + 1, 
-                      mod->rate_matrix->states);
-        fprintf(EXPTOTSUBF, "%12s ", tuplestr);
+      
+      for (node = 0; node < mod->tree->nnodes; node++) {
+	n = lst_get_ptr(tr_postorder(mod->tree), node);
+	if (n == mod->tree) continue;
+	
+	fprintf(EXPTOTSUBF, "Branch above node %d", n->id);
+	if (n->name != NULL && strlen(n->name) > 0) 
+	  fprintf(EXPTOTSUBF, " (leaf labeled '%s')", n->name);
+	fprintf(EXPTOTSUBF, ":\n\n");
+	
+	/* print header */
+	fprintf(EXPTOTSUBF, "%-4s ", "");
+	for (state2 = 0; state2 < mod->rate_matrix->size; state2++) {
+	  get_tuple_str(tuplestr, state2, mod->order + 1, 
+			mod->rate_matrix->states);
+	  fprintf(EXPTOTSUBF, "%12s ", tuplestr);
+	}
+	fprintf(EXPTOTSUBF, "\n");
+	for (state = 0; state < mod->rate_matrix->size; state++) {
+	  get_tuple_str(tuplestr, state, mod->order + 1, 
+			mod->rate_matrix->states);
+	  fprintf(EXPTOTSUBF, "%-4s ", tuplestr);
+	  for (state2 = 0; state2 < mod->rate_matrix->size; state2++) 
+	    fprintf(EXPTOTSUBF, "%12.2f ", 
+		    mod->tree_posteriors->expected_nsubst_tot[0][state][state2][n->id]);
+	  fprintf(EXPTOTSUBF, "\n");
+	}
+	fprintf(EXPTOTSUBF, "\n\n");
       }
-      fprintf(EXPTOTSUBF, "\n");
-      for (state = 0; state < mod->rate_matrix->size; state++) {
-        get_tuple_str(tuplestr, state, mod->order + 1, 
-                      mod->rate_matrix->states);
-        fprintf(EXPTOTSUBF, "%-4s ", tuplestr);
-        for (state2 = 0; state2 < mod->rate_matrix->size; state2++) 
-          fprintf(EXPTOTSUBF, "%12.2f ", 
-                  mod->tree_posteriors->expected_nsubst_tot[0][state][state2][n->id]);
-        fprintf(EXPTOTSUBF, "\n");
-      }
-      fprintf(EXPTOTSUBF, "\n\n");
+      fclose(EXPTOTSUBF);
     }
-    fclose(EXPTOTSUBF);
   }
 
   tl_free_tree_posteriors(mod, msa, mod->tree_posteriors);
   mod->tree_posteriors = NULL;
   str_free(fname);
+
+  if (results != NULL) {
+    ListOfLists *postprob_lol = lol_new(3);
+    if (dobase_lol != NULL) 
+      lol_push_lol(postprob_lol, dobase_lol, "bybase");
+    if (exp_nsub_lol != NULL) 
+      lol_push_lol(postprob_lol, exp_nsub_lol, "exp.nsub");
+    if (exp_nsub_tot_lol != NULL) 
+      lol_push_lol(postprob_lol, exp_nsub_tot_lol, "tot.exp.nsub");
+    lol_push_lol(results, postprob_lol, "post.prob");
+  }
 }
 
 
 void print_window_summary(FILE* WINDOWF, List *window_coords, int win, 
                           int cat, TreeModel *mod, double *gc,
                           int ninf_sites, int nseqs, int header_only) {
-  int j;
+  int j, i;
   if (header_only) {
-    fprintf(WINDOWF, "%5s %8s %8s %4s", "win", "beg", "end", "cat");
-    fprintf(WINDOWF, " %6s", "GC");
-    fprintf(WINDOWF, " %7s", "ninf");
-    fprintf(WINDOWF, " %7s\n", "t");
+    if (WINDOWF != NULL) {
+      fprintf(WINDOWF, "%5s %8s %8s %4s", "win", "beg", "end", "cat");
+      for (i=0; i < nseqs; i++) {
+	char temp[100];
+	sprintf(temp, "GC%i", i);
+	fprintf(WINDOWF, " %6s", temp);
+      }
+      fprintf(WINDOWF, " %7s", "ninf");
+      fprintf(WINDOWF, " %7s\n", "t");
+    }
   }
   else {
-    fprintf(WINDOWF, "%5d %8d %8d %4d", win/2+1, 
-            lst_get_int(window_coords, win), 
-            lst_get_int(window_coords, win+1), cat);
-    fprintf(WINDOWF, " %6.4f", 
-            vec_get(mod->backgd_freqs, 
-                           mod->rate_matrix->inv_states[(int)'G']) + 
-            vec_get(mod->backgd_freqs, 
-                           mod->rate_matrix->inv_states[(int)'C']));
-    for (j = 0; j < nseqs; j++) fprintf(WINDOWF, " %6.4f", gc==NULL ? -1.0 : gc[j]);
-    fprintf(WINDOWF, " %7d", ninf_sites);
-    fprintf(WINDOWF, " %7.4f\n", tr_total_len(mod->tree));
+    if (WINDOWF != NULL) {
+      fprintf(WINDOWF, "%5d %8d %8d %4d", win/2+1, 
+	      lst_get_int(window_coords, win), 
+	      lst_get_int(window_coords, win+1), cat);
+      fprintf(WINDOWF, " %6.4f", 
+	      vec_get(mod->backgd_freqs, 
+		      mod->rate_matrix->inv_states[(int)'G']) + 
+	      vec_get(mod->backgd_freqs, 
+		      mod->rate_matrix->inv_states[(int)'C']));
+      for (j = 0; j < nseqs; j++) 
+	fprintf(WINDOWF, " %6.4f", gc==NULL ? -1.0 : gc[j]);
+      fprintf(WINDOWF, " %7d", ninf_sites);
+      fprintf(WINDOWF, " %7.4f\n", tr_total_len(mod->tree));
+    }
   }
 }
 
@@ -388,7 +557,6 @@ int run_phyloFit(struct phyloFit_struct *pf) {
   int i, j, win, root_leaf_id = -1;
   String *mod_fname;
   MSA *source_msa;
-  FILE *logf = NULL;
   String *tmpstr = str_new(STR_SHORT_LEN);
   List *cats_to_do=NULL;
   double *gc=NULL;
@@ -405,23 +573,16 @@ int run_phyloFit(struct phyloFit_struct *pf) {
   int quiet = pf->quiet;
   TreeModel *input_mod = pf->input_mod;
 
+  if (pf->no_freqs)
+    pf->init_backgd_from_data = FALSE;
 
-  
-  //    print_phyloFit_struct(pf);
-
-  if (pf->log_fname != NULL) {
-    if (!strcmp(pf->log_fname, "-"))
-      logf = stderr;
-    else
-      logf = fopen_fname(pf->log_fname, "w+");
-  }
   if (pf->parsimony_cost_fname != NULL)
     parsimony_cost_file = fopen_fname(pf->parsimony_cost_fname, "w");
 
   if (pf->use_conditionals && pf->use_em) 
     die("ERROR: Cannot use --markov with --EM.    Type %s for usage.\n",
 	pf->see_for_help);
-
+  
   if (pf->likelihood_only && input_mod == NULL)  
     die("ERROR: --lnl requires --init-model.  Type '%s' for usage.\n",
 	pf->see_for_help);
@@ -475,7 +636,7 @@ int run_phyloFit(struct phyloFit_struct *pf) {
       tree = tr_new_from_string(tmpchstr);
       free_tree = TRUE;
     }
-    else if (msa->nseqs == 3 && tm_is_reversible(subst_mod)) {
+    else if (msa->nseqs == 3 && subst_mod_is_reversible(subst_mod) && pf->alt_mod_str == NULL) {
       sprintf(tmpchstr, "(%s,(%s,%s))", msa->names[0], msa->names[1], 
               msa->names[2]);
       tree = tr_new_from_string(tmpchstr);
@@ -488,13 +649,55 @@ int run_phyloFit(struct phyloFit_struct *pf) {
   /* allow for specified ancestor */
   if (pf->root_seqname != NULL) {
     TreeNode *rl;
-    if (tree == NULL || tm_is_reversible(subst_mod)) 
+    if (tree == NULL || subst_mod_is_reversible(subst_mod)) 
       die("ERROR: --ancestor requires --tree and a non-reversible model.\n");
     rl = tr_get_node(tree, pf->root_seqname);     
     if (rl == NULL || rl->parent != tree) 
       die("ERROR: Sequence specified by --ancestor must be a child of the root.\n");
     root_leaf_id = rl->id;
   }
+
+  if (pf->label_str != NULL || pf->label_type != NULL) {
+    if (pf->label_str == NULL || pf->label_type == NULL ||
+	lst_size(pf->label_str) != lst_size(pf->label_type))
+      //shouldn't happen unless bug
+      die("label_str and label_type should both be lists of same length\n");
+    for (i=0; i < lst_size(pf->label_str); i++) {
+      String *currstr = (String*)lst_get_ptr(pf->label_str, i), *arg1, *label;
+      List *tmplst = lst_new_ptr(10);
+      String *nodename;
+      str_split(currstr, ":", tmplst);
+      if (lst_size(tmplst) != 2) 
+	die("ERROR: bad argument to --label-branches or --label-subtree.\n");
+      arg1 = lst_get_ptr(tmplst, 0);
+      label = lst_get_ptr(tmplst, 1);
+      lst_clear(tmplst);
+      if (lst_get_int(pf->label_type, i) == BRANCH_TYPE) {
+	str_split(arg1, ",", tmplst);
+	for (j=0; j < lst_size(tmplst); j++) {
+	  nodename = (String*)lst_get_ptr(tmplst, j);
+	  tr_label_node(tree, nodename->chars, label->chars);
+	}
+	lst_free_strings(tmplst);
+      } else if (lst_get_int(pf->label_type, i) == SUBTREE_TYPE) {
+	int include_leading_branch = FALSE;
+	TreeNode *node;
+	nodename = arg1;
+	node = tr_get_node(tree, nodename->chars);
+	if (node == NULL && nodename->chars[nodename->length-1] == '+') {
+	  nodename->chars[--nodename->length] = '\0';
+	  node = tr_get_node(tree, nodename->chars);
+	  include_leading_branch = TRUE;
+	}
+	tr_label_subtree(tree, nodename->chars, include_leading_branch, 
+			 label->chars);
+      } else die("ERROR got label_type %i\n", lst_get_int(pf->label_type, i));
+      str_free(arg1);
+      str_free(label);
+      lst_free(tmplst);
+    }
+  }
+
   
   if (msa_alph_has_lowercase(msa)) msa_toupper(msa); 
   msa_remove_N_from_alph(msa);    /* for backward compatibility */
@@ -577,15 +780,14 @@ int run_phyloFit(struct phyloFit_struct *pf) {
   }
   if (pf->window_coords != NULL) {
     /* set up summary file */
-    String *sumfname;
+    String *sumfname=NULL;
     msa_coord_map *map;
-
-    if (pf->output_fname_root == NULL) 
-      die("Currently need to specify output file to run phyloFit in windows");
-    sumfname = str_new_charstr(pf->output_fname_root);
-    str_append_charstr(sumfname, ".win-sum");
-    WINDOWF = fopen_fname(sumfname->chars, "w+");
-    str_free(sumfname);
+    if (pf->output_fname_root != NULL) {
+      sumfname = str_new_charstr(pf->output_fname_root);
+      str_append_charstr(sumfname, ".win-sum");
+      WINDOWF = fopen_fname(sumfname->chars, "w+");
+      str_free(sumfname);
+    } 
     print_window_summary(WINDOWF, NULL, 0, 0, NULL, NULL, 0, 0, TRUE);
     
     /* map to coord frame of alignment */
@@ -643,9 +845,20 @@ int run_phyloFit(struct phyloFit_struct *pf) {
 		  pf->rate_consts, NULL);
       }
 
-      mod->noopt_arg = pf->nooptstr;
+      if (pf->use_selection) {
+	mod->selection_idx = 0;
+	mod->selection = pf->selection;
+      }
+
+      mod->noopt_arg = pf->nooptstr == NULL ? NULL : str_new_charstr(pf->nooptstr->chars);
       mod->eqfreq_sym = pf->symfreq;
-      mod->bound_arg = pf->bound_arg;
+      if (pf->bound_arg != NULL) {
+	mod->bound_arg = lst_new_ptr(lst_size(pf->bound_arg));
+	for (j=0; j < lst_size(pf->bound_arg); j++) {
+	  String *tmp = lst_get_ptr(pf->bound_arg, j);
+	  lst_push_ptr(mod->bound_arg, str_new_charstr(tmp->chars));
+	}
+      } else mod->bound_arg = NULL;
 
       mod->use_conditionals = pf->use_conditionals;
 
@@ -778,7 +991,7 @@ int run_phyloFit(struct phyloFit_struct *pf) {
             fprintf(F, "%d\t%.6f\n", j, col_log_probs[j]);
           fclose(F);
           str_free(colprob_fname);
-          free(col_log_probs);
+          sfree(col_log_probs);
         }
       }
       else {                    /* fit model */
@@ -788,13 +1001,13 @@ int run_phyloFit(struct phyloFit_struct *pf) {
             fprintf(stderr, "Extracting sufficient statistics ...\n");
           ss_from_msas(msa, mod->order+1, 0, 
                        pf->cats_to_do_str != NULL ? cats_to_do : NULL, 
-                       NULL, NULL, -1);
+                       NULL, NULL, -1, subst_mod_is_codon_model(mod->subst_mod));
           /* (sufficient stats obtained only for categories of interest) */
       
           if (msa->length > 1000000) { /* throw out original data if
                                           very large */
-            for (j = 0; j < msa->nseqs; j++) free(msa->seqs[j]);
-            free(msa->seqs);
+            for (j = 0; j < msa->nseqs; j++) sfree(msa->seqs[j]);
+            sfree(msa->seqs);
             msa->seqs = NULL;
           }
         }
@@ -809,7 +1022,7 @@ int run_phyloFit(struct phyloFit_struct *pf) {
 	if (pf->init_parsimony)
 	  tm_params_init_branchlens_parsimony(params, mod, msa, cat);
 
-        if (input_mod != NULL && mod->backgd_freqs != NULL && !pf->no_freqs) {
+        if (input_mod != NULL && mod->backgd_freqs != NULL && !pf->no_freqs && pf->init_backgd_from_data) {
           /* in some cases, the eq freqs are needed for
              initialization, but now they should be re-estimated --
              UNLESS user specifies --no-freqs */
@@ -829,14 +1042,12 @@ int run_phyloFit(struct phyloFit_struct *pf) {
           fprintf(stderr, "Fitting tree model to %s using %s%s ...\n",
                   tmpstr->chars, tm_get_subst_mod_string(subst_mod),
                   mod->nratecats > 1 ? " (with rate variation)" : "");
-          if (pf->log_fname != NULL)
-            fprintf(stderr, "(writing log to %s)\n", pf->log_fname);
         }
 
         if (pf->use_em)
-          tm_fit_em(mod, msa, params, cat, pf->precision, logf);
+          tm_fit_em(mod, msa, params, cat, pf->precision, pf->logf);
         else
-          tm_fit(mod, msa, params, cat, pf->precision, logf);
+          tm_fit(mod, msa, params, cat, pf->precision, pf->logf, pf->quiet);
 
         if (pf->error_fname != NULL)
 	  tm_variance(mod, msa, params, cat, pf->error_fname, i!=0 || win!=0);
@@ -870,40 +1081,34 @@ int run_phyloFit(struct phyloFit_struct *pf) {
 	tm_print(F, mod);
 	fclose(F);
       }
-      if (pf->estimated_models != NULL) {
-	lst_push_ptr(pf->estimated_models, (void*)tm_create_copy(mod));
-      }
-      if (pf->model_labels != NULL) {
-	lst_push_ptr(pf->model_labels, (void*)str_new_charstr(mod_fname->chars));
-      }
+      if (pf->results != NULL)
+	lol_push_treeModel(pf->results, mod, mod_fname->chars);
 
       /* output posterior probabilities, if necessary */
-      if (pf->output_fname_root != NULL) {
-	if (pf->do_bases || pf->do_expected_nsubst || pf->do_expected_nsubst_tot) 
-	  print_post_prob_stats(mod, msa, pf->output_fname_root, 
-				pf->do_bases, pf->do_expected_nsubst, 
-				pf->do_expected_nsubst_tot, 
-				cat, quiet);
+      if (pf->do_bases || pf->do_expected_nsubst || pf->do_expected_nsubst_tot) {
+	print_post_prob_stats(mod, msa, pf->output_fname_root, 
+			      pf->do_bases, pf->do_expected_nsubst, 
+			      pf->do_expected_nsubst_tot, 
+			      cat, quiet, NULL);
       }
 
       /* print window summary, if window mode */
       if (pf->window_coords != NULL) {
-	if (gc ==NULL) {
-	  int i, j, total=0;
-	  char c;
+	int i, j, total=0;
+	char c;
+	if (gc == NULL)
 	  gc = smalloc(msa->nseqs*sizeof(double));
-	  for (i=0; i < msa->nseqs; i++) {
-	    total=0;
-	    gc[i]=0;
-	    for (j=0; j<msa->length; j++) {
-	      c = msa_get_char(msa, i, j);
-	      if ((!msa->is_missing[(int)c]) && c != GAP_CHAR) {
-		total++;
-		if (c=='C' || c=='G') gc[i]++;
-	      }
+	for (i=0; i < msa->nseqs; i++) {
+	  total=0;
+	  gc[i]=0;
+	  for (j=0; j<msa->length; j++) {
+	    c = msa_get_char(msa, i, j);
+	    if ((!msa->is_missing[(int)c]) && c != GAP_CHAR) {
+	      total++;
+	      if (c=='C' || c=='G') gc[i]++;
 	    }
-	    gc[i] /= (double)total;
 	  }
+	  gc[i] /= (double)total;
 	}
         print_window_summary(WINDOWF, pf->window_coords, win, cat, mod, gc, 
                              ninf_sites, msa->nseqs, FALSE);
@@ -916,7 +1121,7 @@ int run_phyloFit(struct phyloFit_struct *pf) {
       msa_free(msa);
   }
   if (parsimony_cost_file != NULL) fclose(parsimony_cost_file); 
-  free(mod_fname);
+  str_free(mod_fname);
   str_free(tmpstr);
   if (free_cm) {
     cm_free(pf->cm);
@@ -935,6 +1140,6 @@ int run_phyloFit(struct phyloFit_struct *pf) {
     pf->window_coords = NULL;
   }
   if (gc != NULL)
-    free(gc);
+    sfree(gc);
   return 0;
 }
