@@ -18,7 +18,6 @@ Last updated: 12/14/08
 #include <stdlib.h>
 #include <stdio.h>
 #include <msa.h>
-#include <string.h>
 #include <getopt.h>
 #include <ctype.h>
 #include <misc.h>
@@ -29,21 +28,75 @@ Last updated: 12/14/08
 #include <hmm.h>
 #include <category_map.h>
 #include <gff.h>
+#include <list_of_lists.h>
+#include <matrix.h>
+#include <sufficient_stats.h>
+#include <phylo_fit.h>
+#include <rph_util.h>
 
 #include <Rdefines.h>
 #include <R_ext/Random.h>
 
 void rph_msa_free(SEXP msaP) {
   MSA *msa;
-  msa = (MSA*)EXTPTR_PTR(msaP); fflush(stdout);
+  msa = (MSA*)EXTPTR_PTR(msaP);
   msa_free(msa);
 }
 
 
+void rph_msa_protect_ss(MSA_SS *ss) {
+  int i;
+  rph_protect_mem(ss);
+  if (ss->col_tuples != NULL) {
+    rph_protect_mem(ss->col_tuples);
+    for (i=0; i < ss->ntuples; i++) 
+      rph_protect_mem(ss->col_tuples[i]);
+  }
+  if (ss->tuple_idx != NULL)
+    rph_protect_mem(ss->tuple_idx);
+  if (ss->counts != NULL)
+    rph_protect_mem(ss->counts);
+  if (ss->cat_counts != NULL) {
+    rph_protect_mem(ss->cat_counts);
+    for (i=0; i < ss->msa->ncats; i++)
+      rph_protect_mem(ss->cat_counts[i]);
+  }
+}
+
+void rph_msa_protect(MSA *msa) {
+  int i;
+  rph_protect_mem(msa);
+  if (msa->alphabet != NULL)
+    rph_protect_mem(msa->alphabet);
+  if (msa->names != NULL) {
+    rph_protect_mem(msa->names);
+    for (i=0; i < msa->nseqs; i++)
+      rph_protect_mem(msa->names[i]);
+  }
+  if (msa->seqs != NULL) {
+    rph_protect_mem(msa->seqs);
+    for (i=0; i < msa->nseqs; i++)
+      rph_protect_mem(msa->seqs[i]);
+  }
+  if (msa->categories != NULL)
+    rph_protect_mem(msa->categories);
+  if (msa->ss != NULL) {
+    if (msa != msa->ss->msa) {
+      die("ss pointer mismatch\n");
+    }
+    rph_msa_protect_ss(msa->ss);
+  }
+  if (msa->is_informative != NULL)
+    rph_protect_mem(msa->is_informative);
+}
+
+
 SEXP rph_msa_new_extptr(MSA *msa) {
   SEXP result;
+  rph_msa_protect(msa);
   PROTECT(result=R_MakeExternalPtr((void*)msa, R_NilValue, R_NilValue));
   R_RegisterCFinalizerEx(result, rph_msa_free, 1);
+  rph_msa_protect(msa);
   UNPROTECT(1);
   return result;
 }
@@ -85,26 +138,14 @@ SEXP rph_msa_new(SEXP seqsP, SEXP namesP, SEXP nseqsP, SEXP lengthP,
     strcpy(alphabet, CHARACTER_VALUE(alphabetP));
   }
 
-  /*  printf("nseqs=%i length=%i\n", nseqs, length);
-  if (names != NULL) {
-    for (i=0; i<nseqs; i++) 
-      printf("names[%i]=%s\n", i, names[i]==NULL ? "NULL" : names[i]);
-  }
-  else printf("names is NULL\n");
-  if (seqs != NULL) {
-    for (i=0; i<nseqs; i++)
-      printf("seqs[%i]=%s\n", i, seqs[i]==NULL ? "NULL" : seqs[i]);
-  } else printf("seqs is NULL\n");
-  if (alphabet != NULL) printf("alphabet=%s\n", alphabet);
-  else printf("alphabet is NULL\n");*/
   msa = msa_new(seqs, names, nseqs, length, alphabet);
   if (msa->length > 0 && ! ordered) {
-    ss_from_msas(msa, 1, 0, NULL, NULL, NULL, -1);
+    ss_from_msas(msa, 1, 0, NULL, NULL, NULL, -1, 0);
   } else if (idxOffsetP != R_NilValue)
     msa->idx_offset = INTEGER_VALUE(idxOffsetP);
 
   //don't free seqs or names because they are used by MSA object
-  if (alphabet != NULL) free(alphabet);
+  if (alphabet != NULL) sfree(alphabet);
   
   PROTECT(result = rph_msa_new_extptr(msa));
   numProtect++;
@@ -176,8 +217,8 @@ SEXP rph_msa_reduce_to_4d(SEXP msaP, SEXP gffP) {
   
   ss_reduce_tuple_size(msa, tuple_size);
 
-  str_free(fourD_refseq);
-  cm_free(cm);
+  rph_msa_protect(msa);
+  rph_gff_protect(gff);
   return msaP;
 }
 
@@ -217,7 +258,7 @@ SEXP rph_msa_extract_feature(SEXP msaP, SEXP gffP) {
       }
     }
     ss_remove_zero_counts(msa);
-    free(msa->ss->tuple_idx);
+    sfree(msa->ss->tuple_idx);
     msa->ss->tuple_idx = NULL;
   }
   if (msa->seqs != NULL) {
@@ -237,14 +278,16 @@ SEXP rph_msa_extract_feature(SEXP msaP, SEXP gffP) {
   msa_free_categories(msa);
   msa_update_length(msa);
   msa->idx_offset = 0;
+  rph_msa_protect(msa);
+  rph_gff_protect(gff);
   return msaP;
 }
 
 SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP, 
 		  SEXP do4dP, SEXP alphabetP, 
 		  SEXP tupleSizeP, SEXP refseqP, SEXP orderedP,
-		  SEXP docatsP, SEXP idxOffsetP, SEXP seqnamesP,
-		  SEXP discardSeqnamesP) {
+		  SEXP catsCycleP, SEXP docatsP, SEXP idxOffsetP, 
+		  SEXP seqnamesP, SEXP discardSeqnamesP) {
   int do4d, tupleSize=1, ordered, i;
   GFF_Set *gff=NULL;
   msa_format_type fmt;
@@ -252,7 +295,7 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
   MSA *msa;
   FILE *infile, *refseq=NULL;
   char *alphabet = NULL, *reverse_groups_tag=NULL;
-  int numProtect=0, seq_keep=0;
+  int numProtect=0, seq_keep=0, cycle_size=-1;
   List *cats_to_do=NULL, *cats_to_do_str, *seqnames;
   String *fourD_refseq = NULL;
 
@@ -270,6 +313,10 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
   if (gffP != R_NilValue) 
     gff = (GFF_Set*)EXTPTR_PTR(gffP);
   ordered = INTEGER_VALUE(orderedP);
+
+  if (catsCycleP != R_NilValue)
+    cycle_size = INTEGER_VALUE(catsCycleP);
+
   do4d = INTEGER_VALUE(do4dP);
   if (do4d) {
     cm = cm_new_string_or_file("NCATS=6; CDSplus 1-3; CDSminus 4-6");
@@ -289,8 +336,15 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
     }
     cats_to_do = lst_new_int(6);
     for (i=1; i<=6; i++) lst_push_int(cats_to_do, i);
-  }
-  if (docatsP != R_NilValue) {
+  } else if (catsCycleP != R_NilValue && docatsP != R_NilValue) {
+    int *intp;
+    PROTECT(docatsP = AS_INTEGER(docatsP));
+    numProtect++;
+    intp = INTEGER_POINTER(docatsP);
+    cats_to_do = lst_new_int(LENGTH(docatsP));
+    for (i=0; i < LENGTH(docatsP); i++)
+      lst_push_int(cats_to_do, intp[i]);
+  } else if (docatsP != R_NilValue) {
     int numcats, cmStrLen=100;
     char *cmStr;
     PROTECT(docatsP = AS_CHARACTER(docatsP));
@@ -303,7 +357,6 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
     for (i=0; i<LENGTH(docatsP); i++)
       sprintf(cmStr, "%s; %s %i", cmStr, CHAR(STRING_ELT(docatsP, i)), i+1);
     cm = cm_new_string_or_file(cmStr);
-    free(cmStr);
     cats_to_do_str = lst_new_ptr(numcats);
     for (i=0; i<numcats; i++) {
       lst_push_ptr(cats_to_do_str, 
@@ -313,8 +366,6 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
 		   (CHAR(STRING_ELT(docatsP, i))));
     }
     cats_to_do = cm_get_category_list(cm, cats_to_do_str, TRUE);
-    lst_free_strings(cats_to_do_str);
-    lst_free(cats_to_do_str);
   } else if (gff != NULL) {
     if (cm ==  NULL)
       cm = cm_new_from_features(gff);
@@ -344,9 +395,9 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
 
   infile = fopen_fname(CHARACTER_VALUE(filenameP), "r");
   if (fmt == MAF) {  //reads and automatically converts to SS format
-    msa = maf_read_cats_subset(infile, refseq, tupleSize, alphabet, gff, cm, -1, 
-			       ordered, reverse_groups_tag, NO_STRIP, 0, 
-			       cats_to_do, seqnames, seq_keep);
+    msa = maf_read_cats_subset(infile, refseq, tupleSize, alphabet, gff, cm, 
+			       cycle_size, ordered, reverse_groups_tag,
+			       NO_STRIP, 0, cats_to_do, seqnames, seq_keep);
   } else {
     msa = msa_new_from_file(infile, fmt, alphabet);
     if (idxOffsetP != R_NilValue) msa->idx_offset = INTEGER_VALUE(idxOffsetP);
@@ -375,11 +426,16 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
       msa_reverse_compl_feats(msa, gff, NULL);
     }
     msa_label_categories(msa, gff, cm);
+  } else if (fmt != MAF && cycle_size > 0) {
+    msa->categories = (int*)smalloc(msa->length*sizeof(int));
+    msa->ncats = cycle_size;
+    for (i=0; i < msa->length; i++)
+      msa->categories[i] = (i%cycle_size)+1;
   }
   if (msa == NULL) die("ERROR reading %s\n", CHARACTER_VALUE(filenameP));
   if (tupleSize != 1 || cats_to_do != NULL) {
     if (msa->ss == NULL) {
-      ss_from_msas(msa, tupleSize, ordered, cats_to_do, NULL, NULL, -1);
+      ss_from_msas(msa, tupleSize, ordered, cats_to_do, NULL, NULL, -1, 0);
     }
     else {
       if (msa->ss->tuple_size < tupleSize)
@@ -403,12 +459,6 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
       die("ERROR: rph_msa_read: msa->ss->tuple_idx is not NULL\n");
   if (ordered==0) msa->idx_offset = 0;
 
-  if (cats_to_do != NULL)
-    lst_free(cats_to_do);
-  if (cm != NULL)
-    cm_free(cm);
-  if (alphabet != NULL)
-    free(alphabet);
   if (refseq != NULL)
     fclose(refseq);
   fclose(infile);
@@ -433,10 +483,7 @@ SEXP rph_msa_read(SEXP filenameP, SEXP formatP, SEXP gffP,
   // the subset of the alignment.  
   msa_free_categories(msa);
 
-  if (seqnames != NULL) {
-    lst_free_strings(seqnames);
-    lst_free(seqnames);
-  }
+  if (gff != NULL) rph_gff_protect(gff);
 
   if (numProtect > 0)
     UNPROTECT(numProtect);
@@ -459,7 +506,7 @@ SEXP rph_msa_valid_fmt_str(SEXP formatP) {
 
 //print sequence in given format.  If format not valid,
 //use FASTA, but give no warning.
-SEXP rph_msa_printSeq(SEXP msaP, SEXP filenameP, SEXP formatP, 
+SEXP rph_msa_printSeq(SEXP msaP, SEXP fileP, SEXP formatP, 
 		      SEXP prettyPrintP) {
   MSA *msa;
   msa_format_type fmt;
@@ -467,10 +514,11 @@ SEXP rph_msa_printSeq(SEXP msaP, SEXP filenameP, SEXP formatP,
   fmt = msa_str_to_format(CHARACTER_VALUE(formatP));
   if ((int)fmt == -1) 
     fmt = FASTA;
-  if (filenameP != R_NilValue)
-    msa_print_to_file(CHARACTER_VALUE(filenameP), 
+  if (fileP != R_NilValue)
+    msa_print_to_file(CHARACTER_VALUE(fileP), 
 		      msa, fmt, INTEGER_VALUE(prettyPrintP));
   else msa_print(stdout, msa, fmt, INTEGER_VALUE(prettyPrintP));
+  rph_msa_protect(msa);
   return R_NilValue;
 }
 
@@ -495,7 +543,7 @@ SEXP rph_msa_seqs(SEXP msaP) {
     for  (seq = 0; seq < msa->nseqs; seq++) {
       tempseq = ss_get_one_seq(msa, seq);
       SET_STRING_ELT(result, seq, mkChar(tempseq));
-      free(tempseq);
+      sfree(tempseq);
     }
   } else {
     for (seq = 0; seq < msa->nseqs; seq++) { 
@@ -504,6 +552,7 @@ SEXP rph_msa_seqs(SEXP msaP) {
       SET_STRING_ELT(result, seq, mkChar(msa->seqs[seq]));
     }
   }
+  rph_msa_protect(msa);
   UNPROTECT(1);
   return result;
 }
@@ -631,11 +680,11 @@ SEXP rph_msa_square_brackets(SEXP msaP, SEXP rowsP, SEXP colsP) {
       if (rows[i] >= msa->nseqs) 
 	die("invalid row in rph_msa_square_brackets");
     }
-    names[i] = strdup(msa->names[spec]);
+    names[i] = copy_charstr(msa->names[spec]);
 
     if (cols == NULL) {
       if (msa->ss != NULL) seqs[i] = ss_get_one_seq(msa, spec);
-      else seqs[i] = strdup(msa->seqs[spec]);
+      else seqs[i] = copy_charstr(msa->seqs[spec]);
     } else {
       seqs[i] = smalloc((ncol+1)*sizeof(char));
       for (j=0; j<ncol; j++) {
@@ -647,6 +696,7 @@ SEXP rph_msa_square_brackets(SEXP msaP, SEXP rowsP, SEXP colsP) {
       seqs[i][j] = '\0';
     }
   }
+  rph_msa_protect(msa);
   newMsa = msa_new(seqs, names, nrow, ncol, msa->alphabet);
   if (numprotect > 0) UNPROTECT(numprotect);
   return rph_msa_new_extptr(newMsa);
@@ -657,7 +707,7 @@ SEXP rph_msa_sub_alignment(SEXP msaP, SEXP seqsP, SEXP keepP,
 			   SEXP startcolP, SEXP endcolP, 
 			   SEXP refseqNameP) {
   MSA *msa = (MSA*)EXTPTR_PTR(msaP), *subMsa;
-  int numseq, i, keep=1, startcol, endcol, numProtect=0, refseq;
+  int numseq, i, keep=1, startcol, endcol, numProtect=0, refseq, idx_offset=0;
   List *seqlist_str, *l=NULL;
   msa_coord_map *map = NULL;
 
@@ -672,8 +722,6 @@ SEXP rph_msa_sub_alignment(SEXP msaP, SEXP seqsP, SEXP keepP,
     for (i=0; i<numseq; i++)  
       lst_push_ptr(seqlist_str, str_new_charstr(CHAR(STRING_ELT(seqsP, i))));
     l = msa_seq_indices(msa, seqlist_str);
-    lst_free_strings(seqlist_str);
-    lst_free(seqlist_str);
     keep = LOGICAL_VALUE(keepP);
   }
 
@@ -699,23 +747,23 @@ SEXP rph_msa_sub_alignment(SEXP msaP, SEXP seqsP, SEXP keepP,
       if (endcolP != R_NilValue)
 	endcol -= msa->idx_offset;
     }
+    if (refseq == 1) idx_offset = startcol - 1 + msa->idx_offset;
     map = msa_build_coord_map(msa, refseq);
     startcol = msa_map_seq_to_msa(map, startcol);
     if (endcolP != R_NilValue)
       endcol = msa_map_seq_to_msa(map, endcol);
   }
 
-  if (startcol < 0 || startcol > msa->length)
-    die("start column out of range");
-  if (endcol < 0 || endcol > msa->length)
-    die("end column out of range");
-    
+  if (startcol < 1 || startcol > msa->length)
+    die("start column (%i) out of range", startcol);
+  if (endcol < 1 || endcol > msa->length)
+    die("end column (%i) out of range", endcol);
 
   subMsa = msa_sub_alignment(msa, l, keep, startcol-1, endcol);
+  subMsa->idx_offset = idx_offset;
+  rph_msa_protect(msa);
   if (subMsa == NULL)
     die("ERROR rph_msa_sub_alignment got NULL subMsa\n");
-  if (l != NULL) lst_free(l);
-  if (map != NULL) msa_map_free(map);
   if (numProtect > 0) UNPROTECT(numProtect);
   return rph_msa_new_extptr(subMsa);
 }
@@ -744,16 +792,55 @@ SEXP rph_msa_strip_gaps(SEXP msaP, SEXP stripModeP, SEXP allOrAnyGaps) {
      shouldn't */
   if (unordered) {
     if (msa->ss == NULL) 
-      ss_from_msas(msa, 1, 0, NULL, NULL, NULL, -1);
+      ss_from_msas(msa, 1, 0, NULL, NULL, NULL, -1, 0);
     else if (msa->ss->tuple_idx != NULL) {
-      free(msa->ss->tuple_idx);
+      sfree(msa->ss->tuple_idx);
       msa->ss->tuple_idx = NULL;
     }
   }
+  rph_msa_protect(msa);
   return msaP;
 }
 
 
+SEXP rph_msa_postprob(SEXP msaP, SEXP tmP) {
+  MSA *msa = (MSA*)EXTPTR_PTR(msaP);
+  TreeModel *tm = (TreeModel*)EXTPTR_PTR(tmP);
+  ListOfLists *result = lol_new(1);
+  if (msa->ss == NULL) {
+    ss_from_msas(msa, tm->order+1, 0, NULL, NULL, NULL, -1, 0);
+    rph_msa_protect(msa);
+  }
+  print_post_prob_stats(tm, msa, NULL, 1, 0, 0, -1, 1, result);
+  return rph_listOfLists_to_SEXP(result);
+}
+
+
+SEXP rph_msa_exp_subs(SEXP msaP, SEXP tmP) {
+  MSA *msa = (MSA*)EXTPTR_PTR(msaP);
+  TreeModel *tm = (TreeModel*)EXTPTR_PTR(tmP);
+  ListOfLists *result = lol_new(1);
+  if (msa->ss == NULL) {
+    ss_from_msas(msa, tm->order+1, 0, NULL, NULL, NULL, -1, 0);
+    rph_msa_protect(msa);
+  }
+  print_post_prob_stats(tm, msa, NULL, 0, 1, 0, -1, 1, result);
+  return rph_listOfLists_to_SEXP(result);
+}
+
+
+SEXP rph_msa_exp_tot_subs(SEXP msaP, SEXP tmP) {
+  MSA *msa = (MSA*)EXTPTR_PTR(msaP);
+  TreeModel *tm = (TreeModel*)EXTPTR_PTR(tmP);
+  ListOfLists *result = lol_new(1);
+  if (msa->ss == NULL) {
+    ss_from_msas(msa, tm->order+1, 0, NULL, NULL, NULL, -1, 0);
+    rph_msa_protect(msa);
+  }
+  print_post_prob_stats(tm, msa, NULL, 0, 0, 1, -1, 1, result);
+  return rph_listOfLists_to_SEXP(result);
+
+}
 
 SEXP rph_msa_likelihood(SEXP msaP, SEXP tmP, SEXP gffP, SEXP byColumnP) {
   int by_column, force_order=0, i, j, start, end;
@@ -796,7 +883,6 @@ SEXP rph_msa_likelihood(SEXP msaP, SEXP tmP, SEXP gffP, SEXP byColumnP) {
   if (by_column) {
     for (i=0; i<msa->length; i++)
       resultP[i] = log2*col_log_probs[i];
-    free(col_log_probs);
   } else if (gff != NULL) {    
     if (msa->idx_offset != 0) {
       for (i=0; i < lst_size(gff->features); i++) {
@@ -820,10 +906,11 @@ SEXP rph_msa_likelihood(SEXP msaP, SEXP tmP, SEXP gffP, SEXP byColumnP) {
   else resultP[0] = likelihood;
   
   if (force_order) {
-    free(msa->ss->tuple_idx);
+    sfree(msa->ss->tuple_idx);
     msa->ss->tuple_idx = NULL;
   }
-  
+  rph_msa_protect(msa);
+  if (gff != NULL) rph_gff_protect(gff);
   UNPROTECT(1);
   return result;
 }
@@ -834,13 +921,6 @@ struct base_evolve_struct {
   GFF_Set *gff;
 };
 
-//there is a separate finalizer for msa struct so only
-//free the list and array
-void rph_msa_base_evolve_struct_free(SEXP lP) {
-  struct base_evolve_struct *x = (struct base_evolve_struct*)EXTPTR_PTR(lP);
-  free(x);
-}
-
 
 SEXP rph_msa_base_evolve_struct_get_msa(SEXP lP) {
   struct base_evolve_struct *x = (struct base_evolve_struct*)EXTPTR_PTR(lP);
@@ -875,28 +955,27 @@ SEXP rph_msa_base_evolve(SEXP modP, SEXP nsitesP, SEXP hmmP,
     hmm = (HMM*)EXTPTR_PTR(hmmP);
     nstate = hmm->nstates;
     if (LOGICAL_VALUE(getFeaturesP))
-      labels = malloc(nsites*sizeof(int));
+      labels = smalloc(nsites*sizeof(int));
   }
-  mods = malloc(nstate*sizeof(TreeModel*));
+  mods = smalloc(nstate*sizeof(TreeModel*));
   for (i=0; i<nstate; i++) 
     mods[i] = (TreeModel*)EXTPTR_PTR(VECTOR_ELT(modP, i));
   msa = tm_generate_msa(nsites, hmm, mods, labels);
   seqname = msa->names[0];
-  free(mods);
   PutRNGstate();
   if (labels != NULL) {
     rv = smalloc(sizeof(struct base_evolve_struct));
     feats = gff_new_set();
-    names = malloc(nstate*sizeof(char*));
+    names = smalloc(nstate*sizeof(char*));
     PROTECT(names_sexp = GET_NAMES(modP));
     if (names_sexp == R_NilValue) {
       for (i=0; i < nstate; i++) {
 	sprintf(temp, "state%i", i+1);
-	names[i] = strdup(temp);
+	names[i] = copy_charstr(temp);
       }
     } else {
       for (i = 0 ; i < nstate; i++)
-	names[i] = strdup(CHAR(STRING_ELT(names_sexp, i)));
+	names[i] = copy_charstr(CHAR(STRING_ELT(names_sexp, i)));
     }
     currstart = 0;
     currstate = labels[0];
@@ -919,13 +998,9 @@ SEXP rph_msa_base_evolve(SEXP modP, SEXP nsitesP, SEXP hmmP,
 					 GFF_NULL_FRAME,
 					 temp, TRUE);
     lst_push_ptr(feats->features, newfeat);
-    for (i=0; i < nstate; i++)
-      free(names[i]);
-    free(names);
     rv->msa = msa;
     rv->gff = feats;
     PROTECT(result=R_MakeExternalPtr((void*)rv, R_NilValue, R_NilValue));
-    R_RegisterCFinalizerEx(result, rph_msa_base_evolve_struct_free, 1);
     UNPROTECT(2);
     return result;
   }
@@ -939,7 +1014,7 @@ SEXP rph_msa_concat(SEXP aggregate_msaP, SEXP source_msaP) {
   return aggregate_msaP;
 }
 
-SEXP rph_list_new_extptr(List *l);
+SEXP rph_lst_new_extptr(List *l);
 
 SEXP rph_msa_split_by_gff(SEXP msaP, SEXP gffP) {
   MSA *msa, *newmsa;
@@ -954,24 +1029,25 @@ SEXP rph_msa_split_by_gff(SEXP msaP, SEXP gffP) {
   starts = smalloc(lst_size(gff->features)*sizeof(int));
 
   /* convert GFF to coordinate frame of alignment */
-  if (msa->idx_offset != 0)
-    for (i=0; i < lst_size(gff->features); i++) {
-      checkInterruptN(i, 1000);
-      feat = lst_get_ptr(gff->features, i);
-      starts[i] = feat->start;
-      feat->start -= msa->idx_offset;
-      feat->end -= msa->idx_offset;
-    }
+  for (i=0; i < lst_size(gff->features); i++) {
+    checkInterruptN(i, 1000);
+    feat = lst_get_ptr(gff->features, i);
+    starts[i] = feat->start;
+    feat->start -= msa->idx_offset;
+    feat->end -= msa->idx_offset;
+  }
   msa_map_gff_coords(msa, gff, -1, 0, 0, NULL);
 
   for (i=0; i < lst_size(gff->features); i++) {
     feat = lst_get_ptr(gff->features, i);
     newmsa = msa_sub_alignment(msa, NULL, -1, feat->start -1, feat->end);
-    newmsa->idx_offset = starts[i];
+    newmsa->idx_offset = starts[i]-1;
     lst_push_ptr(rvlist, newmsa);
+    rph_msa_protect(newmsa);
   }
-  free(starts);
-  return rph_list_new_extptr(rvlist);
+  rph_gff_protect(gff);
+  rph_msa_protect(msa);
+  return rph_lst_new_extptr(rvlist);
 }
 
 SEXP rph_msaList_get(SEXP listP, SEXP idxP) {
@@ -988,6 +1064,7 @@ SEXP rph_msaList_get(SEXP listP, SEXP idxP) {
 SEXP rph_msa_reverse_complement(SEXP msaP) {
   MSA *msa = (MSA*)EXTPTR_PTR(msaP);
   msa_reverse_compl(msa);
+  rph_msa_protect(msa);
   return msaP;
 }
 
@@ -1022,9 +1099,92 @@ SEXP rph_msa_informative_feats(SEXP msaP,
   feats = msa_get_informative_feats(msa, min_informative, speclist,
 				    refseq, gaps_inf);
 
-  if (speclist != NULL) lst_free(speclist);
+  rph_msa_protect(msa);
   if (numprotect > 0) UNPROTECT(numprotect);
   return rph_gff_new_extptr(feats);
 }
 
 
+SEXP rph_msa_codon_clean(SEXP msaP, SEXP refseqP, SEXP strandP) {
+  MSA *msa = (MSA*)EXTPTR_PTR(msaP);
+  char strand='+';
+  if (strandP != R_NilValue) {
+    if (strcmp("+", CHARACTER_VALUE(strandP))==0);
+    else if (strcmp("-", CHARACTER_VALUE(strandP))==0)
+      strand='-';
+    else die("Unknown strand %s\n", CHARACTER_VALUE(strandP));
+  }
+  msa_codon_clean(msa, CHARACTER_VALUE(refseqP), strand);
+  rph_msa_protect(msa);
+  return msaP;
+}
+
+
+SEXP rph_msa_get_base_freqs_tuples(SEXP msaP, SEXP modP) {
+  MSA *msa = (MSA*)EXTPTR_PTR(msaP);
+  TreeModel *mod = (TreeModel*)EXTPTR_PTR(modP);
+  int nstate, i;
+  SEXP rv;
+  double *doubleP;
+
+  if (msa->ss == NULL) 
+    ss_from_msas(msa, mod->order+1, 0, NULL, NULL, NULL, -1,
+                 subst_mod_is_codon_model(mod->subst_mod));
+  nstate = int_pow(strlen(msa->alphabet), mod->order+1);
+  if (mod->backgd_freqs == NULL) 
+    mod->backgd_freqs = vec_new(nstate);
+  msa_get_base_freqs_tuples(msa, mod->backgd_freqs, mod->order+1, -1);
+  PROTECT(rv = NEW_NUMERIC(nstate));
+  doubleP = NUMERIC_POINTER(rv);
+  for (i=0; i < nstate; i++) 
+    doubleP[i] = vec_get(mod->backgd_freqs, i);
+  rph_msa_protect(msa);
+  UNPROTECT(1);
+  return rv;
+}
+
+
+SEXP rph_msa_fraction_pairwise_diff(SEXP msaP, SEXP seq1P, SEXP seq2P, 
+				    SEXP ignoreMissingP, SEXP ignoreGapsP) {
+  MSA *msa = (MSA*)EXTPTR_PTR(msaP);
+  SEXP rv;
+  double *dp, val;
+  ListOfLists *lol;
+  Matrix *m;
+  int ignore_missing = LOGICAL_VALUE(ignoreMissingP),
+    ignore_gaps = LOGICAL_VALUE(ignoreGapsP), seq1=-1, seq2=-1;
+  
+  
+  if (seq1P != R_NilValue) seq1 = INTEGER_VALUE(seq1P)-1;
+  if (seq2P != R_NilValue) seq2 = INTEGER_VALUE(seq2P)-1;
+
+  if (seq1 != -1 && seq2 != -1) {
+    PROTECT(rv = NEW_NUMERIC(1));
+    dp = NUMERIC_POINTER(rv);
+    dp[0] = msa_fraction_pairwise_diff(msa, seq1, seq2,
+				       ignore_missing, ignore_gaps);
+  } else if (seq1 != -1) {
+    PROTECT(rv = NEW_NUMERIC(msa->nseqs));
+    dp = NUMERIC_POINTER(rv);
+    for (seq2=0; seq2 < msa->nseqs; seq2++)
+      dp[seq2] = msa_fraction_pairwise_diff(msa, seq1, seq2,
+					    ignore_missing, ignore_gaps);
+  } else {
+    m = mat_new(msa->nseqs, msa->nseqs);
+    for (seq1=0; seq1 < msa->nseqs; seq1++) {
+      mat_set(m, seq1, seq1, 0.0);
+      for (seq2=seq1+1; seq2 < msa->nseqs; seq2++) {
+	val = msa_fraction_pairwise_diff(msa, seq1, seq2,
+					 ignore_missing, ignore_gaps);
+	mat_set(m, seq1, seq2, val);
+	mat_set(m, seq2, seq1, val);
+      }
+    }
+    lol = lol_new(1);
+    lol_push_matrix(lol, m, "pairwise.diff");
+    PROTECT(rv = rph_listOfLists_to_SEXP(lol));
+  }
+  rph_msa_protect(msa);
+  UNPROTECT(1);
+  return rv;
+}
